<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:err="http://www.w3.org/2005/xqt-errors"
                xmlns:saxon="http://saxon.sf.net/"
                xmlns:file="http://expath.org/ns/file"
                xmlns:shared="urn:shared-functions"
                xmlns:local="urn:local-functions"
                exclude-result-prefixes="#all"
                extension-element-prefixes="saxon"
                version="2.0">
   <xsl:import href="properties.xslt"/>
   <!-- *********************************************************************************************************** -->
   <xsl:template name="xsl:initial-template" match="/">
      <xsl:try>
         <xsl:variable name="startTime" select="saxon:timestamp()"/>
         <xsl:sequence select="$startTime[shared:always-false()]"/>
         <xsl:sequence select="shared:log('INFO', 'Batch execution starting...')"/>
         <xsl:call-template name="local:create-sts-report-xml"/>
         <xsl:sequence select="shared:log('INFO', 'Batch execution succeeded')"/>
         <xsl:sequence select="shared:log-execution-time($startTime)"/>
         <xsl:catch>
            <xsl:sequence select="shared:log-error($err:description, $err:module, $err:line-number)"/>
            <xsl:sequence select="shared:log('ERROR', 'Batch execution failed - see above output for any hints')"/>
            <xsl:sequence select="error()"/>
         </xsl:catch>
      </xsl:try>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:template name="local:create-sts-report-xml">
      <xsl:call-template name="shared:transform">
         <xsl:with-param name="options" as="map(*)">
            <xsl:map>
               <xsl:map-entry key="'stylesheet-location'" select="'create-sts-report-xml.xslt'"/>
               <xsl:map-entry key="'source-location'" select="shared:path-to-uri($input-folder, 'bulletin-work.xml')"/>
               <xsl:map-entry key="'base-output-uri'" select="shared:path-to-uri($output-folder, 'sts-report.xml')"/>
            </xsl:map>
         </xsl:with-param>
      </xsl:call-template>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
</xsl:stylesheet>
