<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:local="urn:local-functions"
                exclude-result-prefixes="#all"
                version="2.0">
   <xsl:import href="shared-functions.xslt"/>
   <xsl:strip-space elements="*"/>
   <xsl:variable name="display-obstruction-type" select="local:display-obstruction-type()"/>
   <!-- *********************************************************************************************************** -->
   <xsl:template match="bulletinsWork">
      <obstructionGroups>
         <xsl:apply-templates mode="local:new-obstructionGroup" select="//obstructionGroup"/>
      </obstructionGroups>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:template mode="local:new-obstructionGroup" match="obstructionGroup" as="element()?">
<!--      <xsl:if test="position() eq last()">-->
         <obstructionGroup>
            <groupNo>
               <xsl:value-of select="position()"/>
            </groupNo>
            <area>
               <xsl:value-of select="infrastructure/@area"/>
            </area>
            <bnx>
               <xsl:value-of select="ancestor::bulletinWork/@bnx"/>
            </bnx>
            <state>
               <xsl:value-of select="ancestor::bulletinWork/@state"/>
            </state>
            <referenceNumber>
               <xsl:value-of select="ancestor::bulletinWork/@referenceNumber"/>
            </referenceNumber>
            <week>
               <xsl:value-of select="local:display-week(periods/@week)"/>
            </week>
            <year>
               <xsl:value-of select="local:display-year(periods/@week)"/>
            </year>
            <period>
               <xsl:value-of select="local:display-period(periods)"/>
            </period>
            <type>
               <xsl:value-of select="$display-obstruction-type(@type)"/>
            </type>
            <line>
               <xsl:value-of select="infrastructure/@lineName"/>
            </line>
            <infrastructure>
               <xsl:value-of select="local:display-infrastructure(infrastructure)"/>
            </infrastructure>
            <impact>
               <xsl:value-of select="ancestor::bulletinWork/@impact"/>
            </impact>
            <impactComment>
               <xsl:value-of select="ancestor::bulletinWork/@impactComment"/>
            </impactComment>
            <impactHKV/>
            <v1Area/>
            <impactHKM/>
            <remarks/>
            <numberCOLT/>
         </obstructionGroup>
<!--      </xsl:if>-->
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-date" as="xs:string">
      <xsl:param name="value" as="xs:string"/>
      <xsl:sequence select="format-date(xs:date($value),'[D,2]/[M,2]/[Y]')"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-infrastructure" as="xs:string">
      <xsl:param name="infrastructure" as="element()"/>
      <xsl:for-each select="$infrastructure">
         <xsl:message select="concat('Nb of infra items = ', count(*))"/>
         <xsl:variable name="infrastructure-text-line" as="xs:string*">
            <xsl:apply-templates mode="local:display-infrastructure"/>
         </xsl:variable>
         <xsl:message select="concat('Nb of infra text lines = ', count($infrastructure-text-line))"/>
         <xsl:sequence select="string-join($infrastructure-text-line, $EOL)"/>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-obstruction-type" as="map(*)">
      <xsl:map>
         <xsl:map-entry key="'outOfService'" select="'OOS'"/>
         <xsl:map-entry key="'inaccessible'" select="'INA'"/>
      </xsl:map>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-period" as="xs:string">
      <xsl:param name="periods" as="element()"/>
      <xsl:for-each select="$periods">
         <xsl:variable name="period-line" as="xs:string*">
            <xsl:apply-templates mode="local:display-period"/>
         </xsl:variable>
         <xsl:sequence select="string-join($period-line, $EOL)"/>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-time" as="xs:string">
      <xsl:param name="value" as="xs:string"/>
      <xsl:sequence select="format-time(xs:time($value),'[H,2]:[m,2]')"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-week" as="xs:string*">
      <xsl:param name="week" as="xs:string"/>
      <xsl:for-each select="tokenize($week, ' ')">
         <xsl:sequence select="concat('W', substring-after(., '-'))"/>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:display-year" as="xs:string*">
      <xsl:param name="week" as="xs:string"/>
      <xsl:for-each-group select="tokenize($week, ' ')" group-by="substring-before(., '-')">
         <xsl:sort select="current-grouping-key()"/>
         <xsl:sequence select="current-grouping-key()"/>
      </xsl:for-each-group>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:template mode="local:display-infrastructure" match="lineSectionTrackRef" as="xs:string">
      <xsl:sequence select="string-join((@lineName, @lineSectionName, @lineSectionTrackName), ' ')"/>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:template mode="local:display-infrastructure" match="routingRef" as="xs:string">
      <xsl:sequence select="string-join((@ptcarLongNameDutch, @routingShortNameDutch), ' - ')"/>
<!--      <xsl:sequence select="concat(@ptcarLongNameDutch, ' - ', @routingShortNameDutch)"/>-->
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:template mode="local:display-infrastructure" match="trackSectionRef" as="xs:string">
      <xsl:sequence select="string-join((@ptcarLongNameDutch, @ptdesLongNameDutch, @trackSectionShortNameDutch), ' - ')"/>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:template mode="local:display-period" match="period" as="xs:string">
      <xsl:sequence select="concat(local:display-date(@fromDate),' ',local:display-time(@fromTime),' - ',local:display-date(@toDate),' ',local:display-time(@toTime))"/>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
</xsl:stylesheet>
